function [] = BuildFile(state,ratelist,conlist)

%--------------------------------------------------------------------------
% Function used to bulid the expression file and calculate the value for E,
% D, and F given the k value based on input parameter 'K'. The generator
% file name is Expression.m. [E F D] = Expression(K)
%
% USAGE:  
%          function [] = BuildFile(state,denominator);
% 
% IPNUT:  
%           state  -- (string array cell) output from KAMethod.m
%         
%            relalist -- (String array cell) all the rate constants in the
%                         system, from KAMethod.m
%
%            conlist  -- (String array cell) all concentrations showing up 
%                        in the pseudo-first-order rate constants. 
%
% OUTPUT:  none
%
% Also see, KAPattern.m
%-----2008--MCW------------------------------------------------------------
%-- Ver. 1.0 -- F. Qi --
fid = fopen('Expression.m','w'); 

c = '%';
cc = '%%';
if (isempty(conlist))
    fprintf(fid,'function [E F D] = Expression(K)\n');
end

if(~isempty(conlist))
    fprintf(fid,'function [E F D] = Expression(K,Con)\n');
    fprintf(fid,' \n');
    fprintf(fid,'%s\n',cc);
    fprintf(fid,'%s user supplies the input concentration list \n', c);
    len = length(conlist);
    for i = 1 : len
        temp = conlist{i};
        fprintf(fid,'%s = Con(%i);\n',temp,i);
    end
end

fprintf(fid,' \n');
fprintf(fid,'%s\n',cc);
fprintf(fid,'%s user supplies the input parameter K \n', c);
len = length(ratelist);

for i = 1 : len
    temp = ratelist{i};
    fprintf(fid,'%s = K(%i);\n',temp,i);
end

fprintf(fid,' \n');
fprintf(fid,'%s\n',cc);
fprintf(fid,'%s  numerator\n', c);

len = length(state);
for i = 1 : len
    temp = char(state{i});
    loc = strfind(temp, '=');
    fprintf(fid, 'E(%i) = %s \n',i,temp(loc+1:end));
    %fprintf(fid, 'E(%i) = %s \n',i,temp(5:end));
end

fprintf(fid,' \n');
fprintf(fid,'%s\n',cc);
fprintf(fid,'%s denominator \n', c);

fprintf(fid, 'D = E(1) ');
for i = 2 : len - 1
    fprintf(fid, '+ E(%i) ', i);
end
fprintf(fid, '+ E(%s); \n',num2str(len));

% if(iscell(denominator))
%     fprintf(fid, '%s\n', denominator{1});
% else
%     fprintf(fid,'D = %s ',denominator);
% end
% fprintf(fid,' \n');

fprintf(fid,'%s\n',cc);
fprintf(fid,'%s fractions \n', c);

for i = 1 : len
    fprintf(fid,'F(%i) = E(%i)/D;\n',i,i);
end

fclose(fid);




