function Main

%--------------------------------------------------------------------------
% The graphic user interface for K-A method
% 
% CopyRight, 2008, Computational Bioengineering Group
% Bioengineering and Biotechnology Center
% Medical College of Wisconsin
% 
% Also see ReadInput.m, GetLink.m, KAPattern.m, Wang.m, BuildFile.m 
%
% Feng Qi, Yu Han, Ranjan Dash, Daniel Beard.
% Ver. 2.3, 2009-09  
%--------------------------------------------------------------------------

%% ------------ Callback Functions ---------------

% Figure resize function
function figResize(src,evt) %#ok<INUSD,INUSD>
    fpos = get(f,'Position');    
    set(RightPanel,'Position',...
        [fpos(3)*88/120 fpos(4)*0.1/35 fpos(3)*32/120 fpos(4)*35/35])
    set(LeftPanel1,'Position',...
        [0.1 fpos(4)*17.5/35 fpos(3)*44.9/120 fpos(4)*17.5/35]);
    set(LeftPanel2,'Position',...
        [fpos(3)*44.1/120 fpos(4)*17.5/35 fpos(3)*44/120 fpos(4)*17.5/35]); 
    set(LeftPanel3,'Position',...
        [0.1 0.1 fpos(3)*88/120 fpos(4)*17.5/35]);
end
    
function LeftPanelResize1(src,evt) %#ok<INUSD,INUSD>
     bpos = get(LeftPanel1,'Position');
     set(LinkBoxLabel,'Position',...
         [(bpos(3)-10)/2 bpos(4)-1.5 10 1]);
     set(ListBoxLink,'Position',...
         [1 0.1 bpos(3)-4 bpos(4)-2]); 
end

function LeftPanelResize2(src,evt) %#ok<INUSD,INUSD>
    bpos = get(LeftPanel2,'Position');
    set(StoiBoxLabel,'Position',...
         [(bpos(3)-10)/2 bpos(4)-1.5 15 1]);
    set(ListBoxStoi,'Position',...
         [1 0.1 bpos(3)-4 bpos(4)-2]); 
end

function LeftPanelResize3(src,evt) %#ok<INUSD,INUSD>
    bpos = get(LeftPanel3,'Position');
    set(ExpressBoxLabel,'Position',...
        [(bpos(3)-14)/2 bpos(4)-1.5 14 1]);
    set(ListBoxExpress,'Position',...
        [1 0.1 bpos(3)-4 bpos(4)-2]);        
end

% Right panel resize function
function RightPanelResize(src,evt) %#ok<INUSD,INUSD>
    bpos = get(RightPanel,'Position');
    set(ListBoxLabel,'Position',...
        [(bpos(3)-14)/2 bpos(4)-1.5 14 1]);
    set(ListBoxInput,'Position',...
        [1 bpos(4)-3.5-bpos(4)*15/35 bpos(3)-4 bpos(4)*15/35]);
    
    set(ReadFileButton,'Position',...
        [0.1  bpos(4)*2.0/32 12 1.5]);
    set(GenerateButton,'Position',...
        [13.1 bpos(4)*2.0/32 12 1.5]);
    set(OutputSelectBox,'Position',...
        [0.1 bpos(4)*0.1/32, 32, 2]);
    
    set(FileText,'Position',...
        [1 bpos(4)-3.5 30 1.5]);
    
    set(PatternBoxLabel,'Position',...
        [(bpos(3)-25)/2 bpos(4)*20/35-5.0 25 1]);
    set(ListBoxPattern, 'Position',...
        [1 bpos(4)*10/35-5.5 bpos(3)-4 bpos(4)*10/35]); 
end

%% Callback for plot button
function ReadFileButtonCallback(src,evt) %#ok<INUSD,INUSD>
     % Get workspace variables
     % Make the GUI axes current and create plot
     
     [s,p,v] = uigetfile('*.txt','Please select a input file');
     if(v ==1)
         
         % Clear previous results in ListBox
         set(ListBoxInput,'String','');
         set(ListBoxLink, 'String','');
         set(ListBoxStoi, 'String','');
         set(ListBoxExpress,'String','');
         set(ListBoxPattern,'String','');
         
         set(PatternBoxLabel,'String','The Patterns List : ');
         
         filename = [p s];
         fid = fopen(filename,'rt');
         i = 1;
         tline = {};
         while feof(fid) == 0
             tline{i} = fgetl(fid); %#ok<AGROW>
             i = i + 1;
         end
         fclose(fid);
         set(ListBoxInput,'String',tline);
         set(FileText,'String',['Opened file is ' s]);
     end
end

function GenerateButtonCallback(src, evt) %#ok<INUSD,INUSD>
    if(isempty(filename))
        errordlg('Please open/select a input file first!','No Input File Error');       
    else
        file = filename(1:end-4);
        % get the information of the system
        [Nodes Link KineticMatrix] = ReadInput(file);
        [LinkMatrix LinkList] = GetLink(Link); %#ok<NASGU>
          
        NodeNumber = length(Nodes);
        temp = [];
        for i = 1 : NodeNumber -1
            % Link{i,3} is all the link index for the node i
            temp = Wang(temp, Link{i,3}');
        end

        % all the possible pattern list
        Pattern = temp;       
         
        % PatternList = char(Pattern');
        PatternList = num2str(Pattern);
        
        % display the information on the ListBox
        LinkOut = double2cell(LinkMatrix);
        set(ListBoxLink,'String',LinkOut);
        StoiOut = Stoi2cell(KineticMatrix);
        set(ListBoxStoi,'String',StoiOut);
        
        FileFlag = get(OutputSelectBox,'Value');
        
        % generater the KA result file
        if(~isempty(Nodes))       
            [state denominator ratelist conlist] = KAPattern(file, FileFlag); %#ok<NASGU,NASGU>                       
            % display the result on the expression.m list box  
            outputfile = 'expression.m';
            fid = fopen(outputfile, 'rt');
            i = 1;
            outline = {};
            while feof(fid) == 0 
                outline{i} = fgetl(fid); %#ok<AGROW>
                i = i + 1;
            end
            fclose(fid);
            set(ListBoxExpress,'String',outline);
            sub_f = SubMain(LinkMatrix, Pattern); %#ok<SETNU>                                  
        else
            set(ListBoxExpress,'String','');
        end
        
        set(ListBoxPattern,'String',PatternList);
        set(PatternBoxLabel,'String',['The Patterns List : ',num2str(length(Pattern))]);
               
    end
end

%% ------------ GUI layout ---------------

%% Use system background color for GUI components
% clear all;
clc;
warning off all
panelColor = get(0,'DefaultUicontrolBackgroundColor');

filename = '';
PatternList = '';
%global sub_f
sub_f = [];

%% Set up the figure and defaults
f = figure('Units','characters',...
        'Position',[30 30 130 35],...
        'Color',panelColor,...
        'HandleVisibility','callback',...
        'IntegerHandle','off',...
        'Renderer','painters',...
        'Toolbar','none',...
        'MenuBar','none',...
        'NumberTitle','off',...
        'Name','King-Altamen Pattern Generator, Computational Bioengineering @ MCW',...
        'ResizeFcn',@figResize);

%% Create the right side panel
RightPanel = uipanel('bordertype','etchedin',...
    'BackgroundColor',panelColor,...
    'Units','characters',...
    'Position',[88.1 0.1 31.8 34.5],...
    'Parent',f,...
    'ResizeFcn',@RightPanelResize);

%% Create the left up panel1
LeftPanel1 = uipanel('bordertype','etchedin',...
    'Units','characters',...
    'Position', [0.1 17.5 44 17.5],...
    'Parent',f,...
    'ResizeFcn',@LeftPanelResize1);

%% Creat the left up panel2
LeftPanel2 = uipanel('bordertype','etchedin',...
     'Units','characters',...
     'Position',[44.1 17.5 44 17.5],...
     'Parent',f,...
     'ResizeFcn',@LeftPanelResize2);
 
%% Creat the left buttom panel
LeftPanel3 = uipanel('bordertype','etchedin',...
     'Units','characters',...
     'Position',[0.1 0.1 88 17.5],...
     'Parent',f,...
     'ResizeFcn',@LeftPanelResize3);
 
%% add listbox and label for input file display
ListBoxLabel = uicontrol(f,'Style','text','Units','characters',...
        'Position',[9 32.5 14 1],...
        'String','The input file',...
        'BackgroundColor',panelColor,...
        'Parent',RightPanel);
    
ListBoxInput = uicontrol(f,'Style','listbox','Units','characters',...
               'Position',[1 16.5 30 15],...
               'BackgroundColor','white',...
               'Max',10,'Min',1,...
               'FontSize',10,...
               'Parent',RightPanel); 

FileText   = uicontrol(f,'Style','text','Units','characters',...
          'Position',[1 31.5 30 1.5],...
          'Parent',RightPanel,...
          'String',...
          ['Opened File is ',filename]);           
%% add a listbox for Pattern list
PatternBoxLabel = uicontrol(f,'Style','text','Units','characters',...
        'Position',[9 15 25 1],...
        'String','The Patterns List',...
        'BackgroundColor',panelColor,...
        'Parent',RightPanel);
    
ListBoxPattern = uicontrol(f,'Style','listbox','Units','characters',...
               'Position',[1 4.5 30 10],...
               'BackgroundColor','white',...
               'Max',10,'Min',1,...
               'FontSize',10,...
               'Parent',RightPanel); 

      
%% add the LinkMatrix display listbox and label
LinkBoxLabel = uicontrol(f,'Style','text','Units','characters',...
        'Position',[17 16 10 1],...
        'String','LinkMatrix',...
        'BackgroundColor',panelColor,...
        'Parent',LeftPanel1);

ListBoxLink = uicontrol(f,'Style','listbox','Units','characters',...
               'Position',[1 0.1 42 15.5],...
               'BackgroundColor','white',...
               'Max',10,'Min',1,...
               'FontSize',10,...
               'FontWeight','bold',...
               'Parent',LeftPanel1); 

%% add the KineticMatrix display listbox
StoiBoxLabel = uicontrol(f,'Style','text','Units','characters',...
               'Position',[17 16 15 1],...
               'String','KineticMatrix',...
               'BackgroundColor',panelColor,...
               'Parent',LeftPanel2);

ListBoxStoi = uicontrol(f,'Style','listbox','Units','characters',...
              'Position',[1 0.1 42 15.5],...
              'BackgroundColor','white',...
              'Max',10,'Min',1,...
              'FontSize',10,...
              'FontName','Arial',...
              'FontWeight','bold',...
              'Parent',LeftPanel2); 

%% add the expression.m dislpay listbox
ExpressBoxLabel = uicontrol(f,'Style','text','Units','characters',...
                  'Position',[37 16 14 1],...
                  'String','Expression.m',...
                  'BackgroundColor',panelColor,...
                  'Parent',LeftPanel3);

ListBoxExpress = uicontrol(f,'Style','listbox','Units','characters',...
                 'Position',[2 0.1 84 15.5],...
                 'BackgroundColor','white',...
                 'Max',10,'Min',1,...
                 'FontSize',10,...
                 'Parent',LeftPanel3);    
             
%% add open file button, and with Listdlg 
ReadFileButton = uicontrol(f,'Style','pushbutton','Units','characters',...
        'Position',[0.1 2.0 12 1.5],...
        'String','Open',...
        'Parent',RightPanel,...
        'Callback',@ReadFileButtonCallback,...
        'TooltipString','Select/read a input file.');

%% add generation file button
GenerateButton = uicontrol(f,'Style','pushbutton','Units','characters',...
        'Position',[13.1 2.0 12 1.5],...
        'String','KAPattern',...
        'Parent',RightPanel,...
        'Callback',@GenerateButtonCallback,...
        'TooltipString','Generate the KA patterns.');

%% add the check box for output file
OutputSelectBox = uicontrol(f,'Style','checkbox',...
                  'Units','characters',...
                  'String','Generate the MathML page',...
                  'Value',0,...
                  'Position',[0.1 0.1 32 2],...
                  'Parent', RightPanel,...
                  'TooltipString','Generate the MathML page.');
    
%% functions used to transfer double matirx to cell array
function [out] = double2cell(in)
    if(isempty(in))
        out = {};
    else
        temp = int2str(in);
        row = size(in,1);
        for i = 1 : row
            out{i} = temp(i,:); %#ok<AGROW>
        end
    end
end

function [out] = Stoi2cell(in)
    if(isempty(in))
        out = {};
    else
        numericcells = cellfun(@isnumeric,in);
        [row column] = size(in);
        line = '';
        for i = 1 : row
            for j = 1 : column
                if(numericcells(i,j)==1)
                    temp = sprintf('%10g',in{i,j});
                    line = [line temp]; %#ok<AGROW>
                else
                    temp = sprintf('%10s',in{i,j});
                    line = [line temp]; %#ok<AGROW>
                end
            end
            out{i,1} = line; %#ok<AGROW>
            line = '';
        end
    end
end

end 




