function [A] = Wang(B, C)

% -------------------------------------------------------------------------
% Function used to calculate wang algebra result for two sets of pathway
% list string cell. (The number in the list stands the index of the link)
% 
% USAGE:  
%          [A] = wang(B, C)
%
% INPUT:  
%          B -- (Array of Integer Array) input combination of link-number 
%               list string cell
%          C -- (Integer Array) input of number string cell (only one link)
% 
% OUTPUT: 
%          A -- (Array of Integer Array) Wang Algebra result calculate from 
%               input B and C, result is a string cell
%
% Example: 
%          wang([1;2], [2;3]) = [1, 2; 
%                                1, 3; 
%                                2, 3]
%          wang(wang([1;2], [3;4]), [1;4]) =
%                                 [[1,2,3];
%                                  [1,2,4];
%                                  [1,3,4];
%                                  [2,3,4]]                    
%
%----2008--MCW-------------------------------------------------------------
%-- Ver. 1.2 -- F.Qi --

% First operation, begin from the empty B
A = [];
if(isempty(B))
    A = C;
else
    NumLeft = size(B, 1);
    NumRight = size(C, 1);
    for i = 1 : NumLeft
        for j = 1 : NumRight
            temp = MyTimes(B(i, :), C(j, :));
            if(~isempty(temp))

% Comment. The function ismember is time consuming comparing with 'strmatch' 
% Although it does the same thing here. (F. Qi, 04/26)
%                 if(sum(ismember(A, temp,'rows'))==0)
%                     A = [A; temp]; %#ok<AGROW>
%                 else
%                     rowNum = ismember(A, temp,'rows');
%                     A(rowNum,:) = []; %#ok<AGROW>
%                 end
                
                rowNum=strmatch(round(temp),round(A),'exact');
                if(isempty(rowNum))
                   A = [A; temp];    %#ok<AGROW>
                else
                   A(rowNum, :) = []; %#ok<AGROW>
                end
            end
        end
    end
end

A = sortrows(A);

function  [T] = MyTimes(B, C)
%--------------------------------------------------------------------------
% Function used to perform the Wang Algebra Multiplication for two value 
% 
% USAGE: 
%        [T] = MyTimes(B, C)
%
% INPUT: 
%         B  --  (Integer Array)List of index of the links for wang algebra
%                 element 1
%
%         C  --  (Integer)Index of the wang algebra element 2.
%   
% OUTPUT:
%         T  --  (Integer Array)The result of Wang Algebra Multiplication for 
%                 B and C
%
% Example: 
%         MyTimes([1],   [2]) = [1,2]
%         MyTimes([1,2], [2]) = []
%         MyTimes([1,2], [4]) = [1,2,4]
%--------------------------------------------------------------------------

if(sum(find(B==C))>0)
    T = [];
else
    T = [B, C];
end
T = sort(T);
end

end
